# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os, sys, shutil, tarfile, stat
from threading import Thread, Event
from time import sleep
from subprocess import Popen, PIPE
from string import strip, replace
from build_constants import *

sys.path.append('/var/www/html')
os.environ['DJANGO_SETTINGS_MODULE'] ='ovmtb.settings'

from django.core.management import setup_environ
from ovmtb import settings
from ovmtb.templates.models import TemplateProject, HardwareDescription, DiskImage, UserPackage

setup_environ(settings)

class JeOSCommands:

    def __init__(self, project, hardware, logdir, builddir):
        self.disk_list = ['System.img']
        self.project = project
        self.hardware = hardware
        self.logdir = logdir
        self.builddir = builddir
        self.instantiate_cmd = self.instantiateCommand()
        self.disk_cmd = self.diskCommand()
        self.packages_cmd = self.packageCommand()
        self.userpackages_cmd = self.userPackagesCommand()
        self.group_cmd = self.groupCommand()
        self.vn_name = ""

    def getRootSize(self):
        return "-I %s " %(self.hardware.system_disk)

    def getLogParam(self):
        if settings.OVMTB_MODIFYJEOS_DEBUG:
            return "-D -log %s/project_%s_debug.log " %(self.logdir, self.project.id)
        else:
            return " "

    def getDirParam(self):
        return "%s/project_%s" %(self.builddir, self.project.id)

    def getYumParam(self):
        if self.project.yum_repo:
            yum_param = "-r %s " %(self.project.yum_repo.url)
        elif self.project.dvd_repo:
            yum_param = "-l %s " %(self.project.dvd_repo.path_to_ISO)
        else:
            yum_param = ""

        return yum_param

    def writeParamFile(self, filename, list):
            file_path = "%s/%s" %(self.project.upload_dir, filename)

            f = open(file_path, "w")
            selected = list.split()
            for item in selected:
                f.write("%s\n" %(item))
            f.close()

            return file_path

    def instantiateCommand(self):
        img_param = "-i %s " %(self.project.JeOS_image.image_name)

        self.vm_name = replace(self.project.template_name, " ", "_")
        name_param = '-n %s ' %(self.vm_name)

        mem_param = "-mem %s " %(self.hardware.memory)
        cpu_param = "-cpu %s " %(self.hardware.vcpus)

        swap_param = "-S %s " %(self.hardware.swap_space)
        root_param = self.getRootSize()

        if self.hardware.config_script:
            config_param = "-conf %s " %(self.hardware.config_script)
        else:
            config_param = ""

        if self.hardware.clean_up_script:
            clean_param = "-cln %s " %(self.hardware.clean_up_script)
        else:
            clean_param = ""

        log_param = self.getLogParam()
        dir_param = "-dir %s " %(self.getDirParam())
        return "modifyjeos %s %s %s %s %s %s %s %s %s %s" %(dir_param, img_param, name_param, mem_param, cpu_param, swap_param, root_param, config_param, clean_param, log_param)

    def diskCommand(self):
        ret = "modifyjeos -f %s/System.img %s " %(self.getDirParam(), self.getRootSize())
        
        try:
            disk_images = DiskImage.objects.filter(hardware_id__id__exact=self.hardware.id)
            if disk_images:
                for image in disk_images:
                    self.disk_list.append(image.disk_image_file)
                    ret = ret + "-P %s %s " %(image.disk_image_file, image.size)
                    if image.mount_point:
                        ret = ret + image.mount_point + " "
            else:
                # If there are no disk images return None to skip the step
                return None
        except Exception, e:
            return None

        ret += self.getLogParam()

        return ret

    def packageCommand(self):
        if self.project.selected_packages and (self.project.yum_repo or self.project.dvd_repo):
            file_param = "-p %s" %(self.writeParamFile("packages", self.project.selected_packages))
            return "modifyjeos -f %s/System.img %s %s %s -nogpg %s" %(self.getDirParam(), self.getRootSize(), self.getYumParam(), file_param, self.getLogParam()) 
        else:
            return None

    def groupCommand(self):
        if self.project.selected_groups and (self.project.yum_repo or self.project.dvd_repo):
            file_param = "-p %s" %(self.writeParamFile("groups", self.project.selected_groups))
            return "modifyjeos -f %s/System.img %s %s %s -yo groupinstall -nogpg %s" %(self.getDirParam(), self.getRootSize(), self.getYumParam(), file_param, self.getLogParam())
        else:
            return None

    def userPackagesCommand(self):
        packages = UserPackage.objects.filter(template_project__id__exact=self.project.id)
        if packages:
            userfile_path = "%s/additional_packages" %(self.project.upload_dir)
            f = open(userfile_path, "w")
            for package in packages:
                f.write("%s\n" %(os.path.basename(package.package_file._get_path())))
            f.close()

            userfile_param = "-a %s" %(userfile_path)
            dir_param = "-m %s " %(self.project.upload_dir)
            return "modifyjeos -f %s/System.img %s %s %s %s -nogpg %s" %(self.getDirParam(), self.getRootSize(), self.getYumParam(), userfile_param, dir_param, self.getLogParam())
        else:
            return None

    def getCommands(self):
        return [self.instantiate_cmd, self.disk_cmd, self.packages_cmd, self.userpackages_cmd, self.group_cmd]

    def generateTarBall(self, tar_path, build_path, file_name):
        # Now need to tar up the files that were just created
        tf = "%s/project_%s.tgz" %(tar_path, self.project.id)
        tar = tarfile.open(tf, "w:gz")
        for disk in self.disk_list:
            f = "%s/%s" %(self.vm_name, disk)
            tar.add("%s/%s" %(self.getDirParam(), disk), arcname=f.encode('ascii', 'ignore'))
        tar.add("%s/vm.cfg" %(self.getDirParam()), arcname="%s/vm.cfg" %(self.vm_name))
        tar.close()
        os.chmod(tf, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
        self.project.tarfile = "/ovmtb/data/tarballs/project_%s.tgz" %(self.project.id)
        self.project.builddir = "/ovmtb/data/builds/project_%s" %(self.project.id)
        self.project.save()

class JeOSBuildThread(Thread):

    def __init__(self, build_file, builddir, logdir, tardir, stopEvent, project_id):
        Thread.__init__(self)
        self.build_file = build_file
        self.logdir = logdir
        self.builddir = builddir
        self.tardir = tardir
        self.stopEvent = stopEvent
        self.status = BUILD_QUEUED
        self.progress = 0
        self.buildprocess = None
        self.buildlog = open("%s/%s" %(logdir, build_file), "w", 0)
        self.project_id = project_id

    def __checkStopEvent(self):
        if not self.stopEvent.isSet():
            self.stopEvent.wait(0.1)
            if self.stopEvent.isSet():
                return True
            else:
                return False
        else:
            return True

    def __saveLoop(self, project):
        needsToBeSaved = True

        if self.status == BUILD_STOP:
            return

        while needsToBeSaved:
            try:
                project.save()
                needsToBeSaved = False
            except Exception:
                pass

    def run(self):
        self.status = BUILD_RUNNING
        # Get the required information from the database
        project = TemplateProject.objects.get(id=self.project_id)
        hardware = HardwareDescription.objects.get(template_project__id__exact=self.project_id)
        steps = float(6)
        current_step = 0.0
        project.logfile = "%s/%s" %(self.logdir, self.build_file)
        self.__saveLoop(project)

        try:
            # Generate the calls to modifyjeos
            cmds = JeOSCommands(project, hardware, self.logdir, self.builddir)

            # Check that the build has not been cancelled
            if self.__checkStopEvent():
                self.status = BUILD_CANCELLED
                return

            # Clear, make into the builddir
            shutil.rmtree("%s/project_%s" %(self.builddir, project.id), ignore_errors=True)
            os.mkdir("%s/project_%s" %(self.builddir, project.id))

            project.builddir = ""
            project.tarfile = ""
            project.status = "BLD"
            project.status_percent = 0
            self.__saveLoop(project)

            #Loop through all of the modifyjeos commands that need to be run
            for cmd in cmds.getCommands():
                if not cmd:
                    current_step = current_step + 1
                    project.status_percent = int((current_step / steps) * 100)
                    self.__saveLoop(project)
                    continue

                # Check that the build has not been cancelled
                if self.__checkStopEvent():
                    self.status = BUILD_CANCELLED
                    return

                self.buildlog.write("Running command: %s\n" %(cmd))
                self.buildprocess = Popen(cmd, shell=True, bufsize=0, stdout=PIPE, stderr=PIPE, close_fds=False)
                out = self.buildprocess.stdout
                err = self.buildprocess.stderr

                line = out.readline()
                while line:
                    self.buildlog.write(line)
                    line = out.readline()

                # I need to wait on the build process for a little bit
                self.buildprocess.wait()

                # Check for errors
                if not self.buildprocess.returncode == 0:
                    for line in err:
                        self.buildlog.write(strip(line))
                    project.status = "ERRB"
                    project.status_percent = 0
                    self.__saveLoop(project)
                    self.status = BUILD_ERROR
                    return

                current_step = current_step + 1
                project.status_percent = int((current_step / steps) * 100)
                self.__saveLoop(project)

            # Check that the build has not been cancelled
            if self.__checkStopEvent():
                self.status = BUILD_CANCELLED
                return

            #Now call the cmd.GenerateTarBall function to tar everything up
            self.buildlog.write("Tarring files...\n")
            cmds.generateTarBall(self.tardir, self.builddir, self.build_file)

            # Clean up any package list files
            if (os.access("%s/packages" %(project.upload_dir), os.F_OK)):
                os.remove("%s/packages" %(project.upload_dir))
            if (os.access("%s/additional_packages" %(project.upload_dir), os.F_OK)):
                os.remove("%s/additional_packages" %(project.upload_dir))
            if (os.access("%s/groups" %(project.upload_dir), os.F_OK)):
                os.remove("%s/groups" %(project.upload_dir))

            project.status = "BCOM"
            project.status_percent = 100
            self.__saveLoop(project)

            self.buildlog.write("Tarred files up as: %s" %(project.tarfile))
            self.buildlog.flush()
            self.buildlog.close()
            self.status = BUILD_COMPLETE
        except Exception, e:
            self.buildlog.write("Got an Exception: %s " %(e))
            self.buildlog.flush()
            self.buildlog.close()
            project.status = "ERRB"
            project.status_percent = 0
            project.tarfile = ""
            project.builddir = ""
            self.__saveLoop(project)
            self.status = BUILD_ERROR
            return

if __name__ == "__main__":
    stopEvent = Event()
    qm = QueueMaster(stopEvent)
    qm.start()
    qm.join()
