# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os, sys, shutil, tarfile, logging, logging.handlers, re
from threading import Thread, Event
from time import sleep
from subprocess import Popen, PIPE
from string import strip
from build_constants import *

sys.path.append('/var/www/html')
os.environ['DJANGO_SETTINGS_MODULE'] ='ovmtb.settings'
from django.conf import settings
from JeOSBuildThread import JeOSBuildThread
from JeOSCleanThread import JeOSCleanThread
from ovmtb.resources.models import JeOSImage
from ovmtb.templates.models import TemplateProject
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _

class QueueMaster(Thread):

    def __init__(self, stopEvent,
            queueDir=settings.OVMTB_BUILD_QUEUE_DIR,
            buildDir=settings.OVMTB_BUILD_DIR,
            buildlogDir=settings.OVMTB_LOG_DIR,
            tarDir=settings.OVMTB_TAR_DIR,
            uploadDir=settings.OVMTB_UPLOAD_DIR,
            logfile = "%s/ovmtb.log" %(settings.OVMTB_LOG_DIR)):
        Thread.__init__(self)
        self.current_builds = {}
        self.stopEvent = stopEvent
        self.queueDir = queueDir
        self.buildDir = buildDir
        self.buildlogDir = buildlogDir
        self.logfile = logfile
        self.tarDir = tarDir
        self.uploadDir = uploadDir

        # Set up a specific logger with our desired output level
        self.log = logging.getLogger('ovmtb')
        self.log.setLevel(logging.DEBUG)

        # Add the log message handler to the logger
        handler = logging.handlers.RotatingFileHandler(
                self.logfile, backupCount=5)

        self.log.addHandler(handler)
        self.log.info("QueueMaster thread intialized")

    def __checkDir(self, dir, function):
        self.log.info("Checking that the %s directory exists" %(function))
        if not os.access(dir, os.F_OK):
            try:
                self.log.info("Creating %s directory %s" %(function, dir))
                os.mkdir(dir)
                self.log.info("Using %s as the %s directory" %(dir, function))
            except Exception, e:
                self.log.info("Failed to find or create %s directory %s" %(function, dir))
                return
        else:
            self.log.info("Using %s as the %s directory" %(dir, function))

    def decipherImageName(self, image_name):
        '''Takes a string representing the image name, and attempts to parse it
        according to the modifyjeos pseudo convention <OS><Rel>_<arch>_<class>_jeos
        where:
            - <OS> represents the OS.  'EL' for example.
            - <Rel> is the OS release and update without dots. '47' for OEL 4 U7.
            - <arch> is i386 or x86_64
            - <class> describes whether the machine is Hardware or Para virtualized.
            
        It will return a pair with the first ordinate being a description of the machine
        and the second being the class of the machine.

        :Parameters
            - image_name : The name of the image to decipher
        '''
        description = ""
        type = ""

        os_re = re.compile('RHEL|EL|OEL')
        m = os_re.match(image_name)
        if (m.group() == 'EL') or (m.group() == 'OEL'):
            description += _("Oracle Enterprise Linux ")

        rel_re = re.compile('(?!i386)(?!x86_64)(?P<rel>[0-9])(?P<upd>[0-9])')
        m = rel_re.search(image_name)
        if m:
            description += _("Release %(rel)s Update %(upd)s ") % { 'rel':m.group('rel'), 'upd':m.group('upd') }

        arch_re = re.compile('i386|x86_64')
        m = arch_re.search(image_name)
        if m:
            description += _("for %(arch)s ") % { 'arch' : m.group() }

        type_re = re.compile('HVM|PVM')
        m = type_re.search(image_name)
        if m:
            type = m.group()
            if m.group() == 'HVM':
                description += _("hardware virtualized machine (HVM)")
            else:
                description += _("para-virtualized machine (PVM)")

        if not description:
            descripition = _("User defined image")

        return (description, type)

    def disableAllImages(self):
        '''Disables all images in database.  Used if there is a problem generating a list from
        the modifyjeos command.
        '''
        for image in JeOSImage.objects.all():
            image.active = False
            image.save()
       
    def scanJeOSImages(self):
        '''Runs the command 'modifyjeos -list' in a subprocess, and enters the response
        as images that may be used in template creation.
        '''
        cmd = "modifyjeos -list"

        try:
            scanProcess = Popen(cmd, shell=True, bufsize=0, stdout=PIPE, stderr=PIPE, close_fds=False)
            out = scanProcess.stdout
            err = scanProcess.stderr
        except Exception, e:
            self.disableAllImages()
            return

        line = out.readline()
        while line:
            image_name = line.strip()
            try:
                image = JeOSImage.objects.get(image_name=image_name)
                image.active = True
                image.save()
            except ObjectDoesNotExist:
                (description, type) = self.decipherImageName(image_name)
                self.log.info("Adding image %s" %(image_name))
                image = JeOSImage(image_name=image_name,
                                  type=type,
                                  description=description,
                                  active=True)
                image.save()
            line = out.readline()

    def run(self):
        self.__checkDir(self.queueDir, "queue")
        self.__checkDir(self.buildDir, "build")
        self.__checkDir(self.buildlogDir, "log")
        self.__checkDir(self.tarDir, "tar")
        self.__checkDir(self.uploadDir, "upload")

        self.log.info("Checking images supported by the modifyjeos command")
        self.scanJeOSImages()
        
        self.log.debug("Starting main QueueMaster loop...")
        while 1:
            # Look in spool directory for new build files
            files = os.listdir(self.queueDir)
            for file in files:
                if file == ".svn":
                    continue
                elif file.startswith("build"):
                    self.log.debug("Found build file %s" %(file))
                    if file not in self.current_builds.keys():
                        self.log.info("Adding build %s to currently running builds" %(file))
                        f = open("%s/%s" %(self.queueDir, file), "r")
                        project_id = int(f.read())
                        self.current_builds[file] = JeOSBuildThread(file, self.buildDir, self.buildlogDir, self.tarDir, Event(), project_id)
                elif file.startswith("clean"):
                    self.log.debug("Found clean up file %s" %(file))
                    if file not in self.current_builds.keys():
                        self.log.info("Cleaning up project %s" %(project_id))

                        # First I need to stop any running build threads
                        build_thread = None
                        regex = re.compile("clean")
                        bf = regex.sub("build", file)
                        try:
                            build_thread = self.current_builds[bf]
                            self.log.info("Project to be cleaned has running build %s" %(bf))
                            build_thread.stopEvent.set()
                        except KeyError, e:
                            pass

                        f = open("%s/%s" %(self.queueDir, file), "r")
                        project_id = int(f.read())
                        self.current_builds[file] = JeOSCleanThread(file, project_id, build_thread=build_thread)
                elif file.startswith("delete"):
                    self.log.debug("Found delete file %s" %(file))

                    if file not in self.current_builds.keys():
                        build_thread = None
                        regex = re.compile("delete")
                        bf = regex.sub("build", file)
                        try:
                            build_thread = self.current_builds[bf]
                            f = open("%s/%s" %(self.queueDir, bf), "r")
                            project_id = int(f.read())

                            self.log.info("Project to be deleted has running build %s" %(bf))
                            build_thread.stopEvent.set()
                            self.log.info("Waiting for build thread to stop")
                            build_thread.join()
                            TemplateProject.objects.get(id=project_id).delete()
                        except KeyError, e:
                            pass

                        f = open("%s/%s" %(self.queueDir, file), "r")
                        for line in f.readlines():
                            self.log.debug("Deleting file %s" %(line.strip()))
                            try:
                                shutil.rmtree(line.strip())
                            except Exception:
                                os.remove(line.strip())

                    f.close()
                    try:
                        os.remove("%s/%s" %(self.queueDir, file))
                    except IOError, e:
                        self.log.info("Could not clear delete file")

            # Check build threads for new builds and start them
            builds = self.current_builds.copy()
            for k, t in builds.iteritems():
                if t.status == BUILD_QUEUED:
                    t.start()
                # Build is complete or had an error
                elif t.status == BUILD_COMPLETE or t.status == BUILD_ERROR or t.status == BUILD_CANCELLED:
                    self.log.info("Removing build for %s" %(k))
                    os.remove("%s/%s" %(self.queueDir, k))
                    del self.current_builds[k]

            # Sleep for awhile
            self.stopEvent.wait(10)

            if self.stopEvent.isSet():
                break

        self.log.info("Stopping any builds that are currently running")
        for k, t in self.current_builds.iteritems():
            t.stopEvent.set()

        self.log.debug("Exiting main loop...")
