# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.utils.translation import ugettext as _
from ovmtb import settings
from ovmtb.templates.common import *
import datetime, os

class OVMTBAuditLogger:
    '''Logs an audit trail of access of the OVMTB application.  Implements the
    Django framework middleware API.  For more information see:
    
    http://docs.djangoproject.com/en/dev/topics/http/middleware/
    
    If the value of OVMTB_AUDIT parameter in the settings file is
    True, then requests will be logged to the file given for the
    OVMTB_AUDIT_LOG parameter within the directory specified by
    OVMTB_LOG_DIR. 
    '''
    
    def openLog(self):
        '''Utility function to open the log file.  Default is to open the log in
        the OVMTB_LOG_DIR directory with the name OVMTB_AUDIT_LOG.
        '''
        try:
            filename = "%s/%s" %(settings.OVMTB_LOG_DIR, settings.OVMTB_AUDIT_LOG)
        except AttributeError:
            try:
                filename = "%s/audit.log" %(settings.OVMTB_LOG_DIR)
            except AttributeError:
                filename = "/var/run/ovmtb/audit.log"
            
        self.log = open(filename, "a")

    def writeLog(self, msg):
        '''Utility function that writes the message to the log file, and then
        closes and syncs the log file
        
        :Parameters
            - 'msg': The message to write to the log file
        '''
        self.log.write(msg)
        self.log.flush()
        os.fsync(self.log.fileno())
        self.log.close()
    
    def formatMessage(self, request):
        '''Utility function to format the message.  Incorporates the string
        passed in as the test parameter into the message.
        
        :Parameters
            - 'request': The HttpRequest object from the Django framework
        :Returns
            - A string containing the following message
            <time> <client IP>:<user name> -- <text>
        '''
        now = datetime.datetime.now()
        user = request.user
        client = request.get_host()
        method = request.method
        path = request.get_full_path()
        return "%s Got a %s request for %s from %s:%s\n" %(now, method, path, client, user)
    
    def process_view(self, request, view_func, view_args, view_kwargs):
        try:
            do_log = settings.OVMTB_AUDIT
        except AttributeError:
            do_log = False

        if do_log:
            self.openLog()
            msg = self.formatMessage(request)
            try:
                self.writeLog(msg)
            except IOError, e:
                log_error_message(_("Unable to write to audit log: %(e)s") % {'e':e})
