# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.utils.translation import ugettext as _
from ovmtb.resources.CompsEntityResolver import CompsEntityResolver
from ovmtb.resources.EL4CompsHandler import EL4CompsHandler
from ovmtb.resources.EL5CompsHandler import EL5CompsHandler
from ovmtb.resources.EL5PrimaryHandler import EL5PrimaryHandler
import gzip, xml.sax, os

class NullISOParser:
    '''Abstract class that provides an interface to get a list of packages from an ISO'''

    def __init__(self, path):
        '''Constructor
        
        :Parameters
            - 'path': The full path the location of the expanded ISO on the local file system
        '''
        self.path = path
        
    def generate_list(self):
        '''The NullISOParser class accepts a path to the location of an extracted ISO on the local file system, and
        returns a 2 - tuple with lists of the packages and groups available at the ISO location.
        '''
        return ([], [])

class EL4ISOParser(NullISOParser):
    '''Concrete implementation of the NullISOParser class specifically to handle an OEL 4 ISO'''
    
    def __init__(self, path):
        NullISOParser.__init__(self, path)
        self.compsfile = '%s/Enterprise/base/comps.xml' %(path)
        
    def generate_list(self):
        h = EL4CompsHandler()
    
        parser = xml.sax.make_parser()
        parser.setContentHandler(h)
        parser.setEntityResolver(CompsEntityResolver())
        parser.parse(self.compsfile)
    
        return (h.package_list, h.groups_list)

class EL5ISOParser(NullISOParser):
    '''Concrete implementation of the NullISOParser class specifically to handle an OEL 5 ISO'''
    
    def __init__(self, path):
        NullISOParser.__init__(self, path)

        self.primarylist = ['%s/VT/repodata/primary.xml.gz' %(path),
            '%s/Server/repodata/primary.xml.gz' %(path),
            '%s/Cluster/repodata/primary.xml.gz' %(path),
            '%s/ClusterStorage/repodata/primary.xml.gz' %(path)
        ]
    
        self.compslist = ['%s/Cluster/repodata/comps-rhel5-cluster.xml' %(path),
            '%s/ClusterStorage/repodata/comps-rhel5-cluster-st.xml' %(path),
            '%s/Server/repodata/comps-rhel5-server-core.xml' %(path),
            '%s/VT/repodata/comps-rhel5-vt.xml' %(path)
        ]

    def generate_list(self):
        packages = []
        groups = []
    
        for file in self.primarylist:
            h =  EL5PrimaryHandler()
            primary = gzip.GzipFile(file)
            xml.sax.parse(primary, h)
            packages.extend(h.package_list)
    
        for file in self.compslist:
            h =  EL5CompsHandler()
            parser = xml.sax.make_parser()
            parser.setContentHandler(h)
            parser.setEntityResolver(CompsEntityResolver())
            parser.parse(file)
            groups.extend(h.groups_list)
    
        return (packages, list(set(groups)))

def make_parser(path):
    '''Factory object to create a parser object for the ISO located at 'path'
    
    :Parameters
        - 'path': The full path to the ISO on the local filesystem
    :Returns
        - An instance of the NullISOParser class (or subclass) that can be used to generate a list of packages and groups
    :Raises
        - Nothing
    '''
    el4_indicator = '/Enterprise/base/comps.xml'
    el5_indicator = '/Server/repodata/primary.xml.gz'

    if os.access("%s%s" %(path, el4_indicator), os.R_OK):
        return EL4ISOParser(path)        
    elif os.access("%s%s" %(path, el5_indicator), os.R_OK):
        return EL5ISOParser(path)
    else:
        raise Exception(_("%(p)s does not appear to be a valid DVD ISO") %{'p':path})
