# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect

class FormPage:
    '''Abstract class that has basic logic for displaying a page with a form.

    The logic for handling requests can be abstracted to two basic steps:

    1.  If it is a GET type request, load a template with a form that posts to itself,
    passing in whatever context_params and session data is required.
    2.  If it is a POST request figure out what HTML object was the source of the POST,
    take the appropriate steps, and then redirect to the appropriate URL.

    The FormPage class models these basic steps.  For all requests, the values of instance
    variables define the template and context_params objects that will be passed to the
    django.template.loader that is used to generate the HttpResponse object.

    Subclasses that wish to define specific action related to a particular HTML object POST
    should define a method that takes one parameter, the HttpRequest object passed in from
    the Django framework, and returns either an HttpResponse object, an HttpResponseRedirect
    object, or None.  An entry in the function_map dictionary should be created using the
    string value of the "name" attribute of the HTML object of interest as the key and the
    function as the value.  For example:

    def doSomethingForFoo(self, request):
        # logic here ...

    def __init__(self, ....):
        FormPage.__init__(self, ...)
        self.function_map['Foo'] = self.doSomethingForFoo
        self.urls['Foo'] = "/somewhere/to/go/after/Foo"

    This method will be called when the HttpResponse method is called and the appropriate
    object is in the POST data.  If the method returns something other than none, that will
    be returned to the caller, and no further processing of the POST data will be done.

    If the method needs specific URLs to redirect to for certain cases, these URLs can be
    stored in the urls instance variable.  For example:

    def doSomethingForFoo(self, request):
        # logic here ...
        return django.http.HttpResponseRedirect(self.urls['Foo'])

    def __init__(self, ....):
        FormPage.__init__(self, ...)
        self.function_map['Foo'] = self.doSomethingForFoo
        self.urls['Foo'] = "/somewhere/to/go/after/Foo"

    The FormPage class defines basic functions and urls for HTML button input objects that
    can be used to navigate to the next and previous pages in a theoretical series of pages
    that comprise a wizard.  There are also basic functions for cancelling out of the wizard
    and saving progress.  The mapping is as follows:

    HTML object                       Class function
    saveButton1, saveButton2          saveButton
    cancelButton1, cancelButton2      cancelButton
    nextButton1, nextButton2          nextButton
    previousButton1, previousButton2  previousButton

    Templates should be written to use those values for the "name" atrribute of the button input.
    Subclasses should override these functions.  Currently the functions all redirect to the URL
    "/ovmtb/".

    Data in the request session can be handled be creating a list of tuples with the key to look
    for in the session dictionary and the key to store the value under in the context_params or
    session_data.  The lists should be passed in as the session_keys or context_keys.

    The FormPage class looks for 'info_messages' and 'error_messages' in the session dictionary,
    and stores the values in context_params under the same key if present.
    '''

    def __init__(self, context_params=None, context_keys=None, function_map=None, template=None,
        urls=None, session_keys=None, session_data=None):
        '''Creates a FormPage object.

        :Parameters
            - 'context_params': The context parameters passed into the template.
            - 'context_keys': A list of triples describing what data should be taken from the request
            session and stored in session_data.
            - 'function_map': A dictionary mapping the "name" value of POSTing HTML objects to functions
              that handle the logic for that object.
            - 'template': The template to load.
            - 'urls': A dictionary of URLs that can be used to redirect the user base on the nature of the
              POST data.
            - 'session_keys': A list of triples describing what data should be taken from the request
            session and stored in session_data.
            - 'session_data': A dictionary of extra data from the session
        '''
        if context_params:
            self.context_params = context_params
        else:
            self.context_params = {}

        if function_map:
            self.function_map = function_map
        else:
            self.function_map = {}
            self.function_map['saveButton1'] = self.saveButton
            self.function_map['saveButton2'] = self.saveButton
            self.function_map['cancelButton1'] = self.cancelButton
            self.function_map['cancelButton2'] = self.cancelButton
            self.function_map['nextButton1'] = self.saveButton
            self.function_map['nextButton2'] = self.saveButton
            self.function_map['previousButton1'] = self.cancelButton
            self.function_map['previousButton2'] = self.cancelButton

        if urls:
            self.urls = urls
        else:
            self.urls = {}
            self.urls['cancel'] = '/ovmtb/'
            self.urls['save'] = '/ovmtb/'
            self.urls['previous'] = '/ovmtb/'
            self.urls['next'] = '/ovmtb/'

        if context_keys:
            self.context_keys = context_keys
        else:
            self.context_keys = [('info_messages', 'info_messages'), ('error_messages', 'error_messages')]

        if session_keys:
            self.session_keys = session_keys
        else:
            self.session_keys = []

        self.template = template
        self.session_data = session_data

    def cancelButton(self, request):
        '''Called if the POST HTML object is named 'cancelButton1' or 'cancelButton2'

        :Parameters
            - 'request': The request object passed in from the Django framework.

        :Returns
            - the return of the django.http.HttpResponseRedirect class constructor, which is
            passed self.urls['cancel'].
        '''
        return HttpResponseRedirect(self.urls['cancel'])

    def saveButton(self, request):
        '''Called if the POSTing HTML object is named 'saveButton1' or 'saveButton2'

        :Parameters
            - 'request': The request object passed in from the Django framework, which is
            passed self.urls['save'].

        :Returns
            - the return of the django.http.HttpResponseRedirect class constructor.
        '''
        return HttpResponseRedirect(self.urls['save'])

    def nextButton(self, request):
        '''Called if the POSTing HTML object is named 'nextButton1' or 'nextButton2'

        :Parameters
            - 'request': The request object passed in from the Django framework.

        :Returns
            - the return of the django.http.HttpResponseRedirect class constructor, which is
            passed self.urls['next'].
        '''
        return HttpResponseRedirect(self.urls['next'])

    def previousButton(self, request):
        '''Called if the POSTing HTML object is named 'previousButton1' or 'previousButton2'

        :Parameters
            - 'request': The request object passed in from the Django framework.

        :Returns
            - the return of the django.http.HttpResponseRedirect class constructor, which is
            passed self.urls['previous'].
        '''
        return HttpResponseRedirect(self.urls['previous'])

    def getFunction(self, request):
        '''View logic for GET requests

        This function holds the basic view logic for any GET request.  The
        django.template.loader.get_template object will then be used to load the template, an
        instance of the django.template.RequestConext will be created, populated with the context_params
        dictionary, and request object.  The django.template.loader.render object will be called
        with the django.template.RequestContext and returned using django.http.HttpResponse.

        This is the standard but long winded way to display a view in the Django framework, For more
        information see the Django documentation:

        http://docs.djangoproject.com/en/dev/topics/http/views/#topics-http-views

        :Parameters
            - 'request': The request object passed in from the Django framework.

        :Returns
            - the return of the django.http.HttpResponse class constructor.

        :Throws
            - This could throw many types of exceptions.
        '''
        l = loader.get_template(self.template)
        c = RequestContext(request, self.context_params)
        return HttpResponse(l.render(c))

    def scanSessionData(self, request, keys, dict):
        '''Checks the session for data that should be stored in context_params or session_data.

        :Parameters
            - 'request': The request object passed in from the Django framework.
            - 'keys': A list of tuples containing the key to look for and the key
            to store the value under in the dictionary.
            - 'dict': The dictionary to store the session data in.
        '''
        for pair in keys:
            try:
                value = request.session[pair[0]]
                if not dict:
                    dict = {}
                dict[pair[1]] = value
                del request.session[pair[0]]
            except KeyError, e:
                continue

    def scanSessionDataForContextKeys(self, request):
        '''Checks the session for data that should be stored in context_params.

        :Parameters
            - 'request': The request object passed in from the Django framework.
        '''
        self.scanSessionData(request, self.context_keys, self.context_params)

    def scanSessionDataForSessionKeys(self, request):
        '''Checks the session for data that should be stored in session_data.

        :Parameters
            - 'request': The request object passed in from the Django framework.
        '''
        self.scanSessionData(request, self.session_keys, self.session_data)

    def HttpResponse(self, request):
        '''Generate the approprate django.http.HttpResponse object based on the type of request.'''
        self.scanSessionDataForContextKeys(request)
        self.scanSessionDataForSessionKeys(request)

        if request.method == 'POST':
            for object in request.POST:
                try:
                    ret = self.function_map[object](request)

                    # The method called should either return a subclass of HttpResponse, or None
                    # If an object was returned, that should be returned to the called, otherwise continue
                    if ret:
                        return ret
                except KeyError, e:
                    pass
        else:
            return self.getFunction(request)
        
