# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.core.exceptions import ObjectDoesNotExist
from ovmtb.resources.FormPage import FormPage

class ModifyByIDFormPage(FormPage):
    '''Basic view logic for a page that will be used to add or edit a database object.

    The ModifyByIDFormPage class operates in mostly the same manner as the FormPage class.
    The class requires two additional parameters, a subclass of the django.db.models.Model class and a
    subclass of the django.forms.ModelForm class.  A third optional parameter is the id of the Model
    object in the database.  This assumes that the Model uses the Django framework's automatically
    indexing primary key.

    For a GET type request, an instance of the ModelForm class will be created.  If the id parameter was
    set, the corresponding Model object will be pulled from the database and used to populate the
    ModelForm instance.  The ModelForm instance will then be added to the context_params dictionary at
    the key "form".

    For a POST type request, the same logic used in the FormPage will be used, except the function taken
    from the function_map dictionary will be called with the request, id, and ModelForm objects. 
    '''

    def __init__(self, modelClass=None, modelFormClass=None, context_params=None, function_map=None,
        template=None, urls=None, id=None):
        '''Creates a ModifyByIDFormPage object.

        :Parameters
            - 'modelClass': The subclass of django.db.models.Model that will be accessed from the database. 
            - 'modelFormClass': The subclass of django.froms.ModelForm that will be put into the 
              context_params. 
            - 'context_params': The context parameters passed into the template.
            - 'function_map': A dictionary mapping the "name" value of POSTing HTML objects to functions
              that handle the logic for that object.
            - 'template': The template to load.
            - 'urls': A dictionary of URLs that can be used to redirect the user based on the nature of the
              POST data.
            - 'id': The primary key ID of the database object of the modelClass type.
        '''
        FormPage.__init__(self, context_params=context_params, function_map=function_map, template=template, urls=urls)
        self.modelClass = modelClass
        self.modelFormClass = modelFormClass
        self.id = id
        self.form = None

    def setupModelForm(self, data=None):
        '''Convienence function that setups up an instance of the modelFormClass.  If the 'id' instance variable has been
        set, the modelFormClass will be bound to the object pulled from the databse using that 'id'.  If the data
        parameter to this method is supplied, then the modelFormClass will be bound to that data.  If it is not then the
        modelFormClass will be bound to the values in the 'session_data' instance variable.

        The setupModelForm method does not return any value, but it does alter the state of the object as the
        'form' instance variable will be initialized to a new instance of the modelFormClass and stored in
        'context_params['form']'.
        '''

        # It make look like this can be simplified, but it cannot.  If you supply the data parameter to a ModelForm
        # constructor it performs validation on the data, even it you supply None.  So you cannot pass in data or
        # session_data as None on the first visit to the view, assuming that the ModelForm class will know not to validate.
        if self.id:
            try:
                o = self.modelClass.objects.get(id=self.id)
                if data:
                    self.form = self.modelFormClass(data=data, instance=o)
                elif self.session_data:
                    self.form = self.modelFormClass(data=self.session_data, instance=o)
                else:
                    self.form = self.modelFormClass(instance=o)
            except ObjectDoesNotExist:
                if data:
                    self.form = self.modelFormClass(data=data)
                elif self.session_data:
                    self.form = self.modelFormClass(data=self.session_data)
                else:
                    self.form = self.modelFormClass()
        else:
            if data:
                self.form = self.modelFormClass(data=data)
            elif self.session_data:
                self.form = self.modelFormClass(data=self.session_data)
            else:
                self.form = self.modelFormClass()

        self.context_params['form'] = self.form

    def HttpResponse(self, request):
        '''Differs from the FormPage method in that an instance of the modelFormClass will always be
        created.

        :Parameters
            - 'request': The request object passed in from the Django framework.

        :Returns
            - the return of the django.http.HttpResponse class constructor.

        :Throws
            - This could throw many types of exceptions.
        '''
        self.setupModelForm()
        return FormPage.HttpResponse(self, request)
