# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import ugettext as _

class TableFormPage:
    '''View logic for a page consisting of a table and action buttons.

    The TableFormPage class implements the basic view functionality for a page that will display a
    form that contains a table of objects.  The form will have a table that will list all of the
    objects of a particular subclass of the django.model class currently in the database.  The form
    will have several action buttons.  All buttons will POST to the same base url.  The
    TableFormPage class allows definition of a function map that maps the POST data value for each
    button to a particular object.

    The POST data will be searched for each key in the function_map dictionary.  Once a match has
    been found the function will be called and is expected to return an appropriate HttpResponse object
    that can be used by the Django framework.  Care should be taken not to create a situation where
    the POST data contains multiple HTML objects that need to be handled.  This scenario will not be
    handled by this class and the behavoir of the class will be unpredictable.
    
    The ideal use case is to create a subclass of TableFormPage that defines all of the appropriate
    functions need to handle POST requests, populate with other necessary objects and then call the
    HttpResponse object.  It can also be used directly in a more short hand manner.  See the
    constructor documentation for details.
    '''

    def editButton(self, request):
        '''Logic for handling POST request due to the user clicking the 'Edit' button.
        
        This object handles POST requests the result from the user clicking on the 'Edit' button.  The
        response will be to redirect to the add_edit_url class instance variable.
        
        :Parameters
            - 'request': The request object passed in from the Django framework
            
        :Returns
            - The return value of the django.http.HttpResponseRedirect object
            
        :Raises
            - Many different exceptions could be raised.
        '''
        try:
            if self.edit_url:
                return HttpResponseRedirect('%s%s/' %(self.edit_url, request.POST['radioSelect']))
            else:
                raise Exception(self.noURLMsg)
        except KeyError, e:
            if self.base_url:
                request.session['info_messages'] = [_("Please select an item from the list")]
                return HttpResponseRedirect(self.base_url)
            else:
                raise Exception(self.noURLMsg)

    def deleteButton(self, request, info_msg=None):
        '''Logic for handling POST request due to the user clicking the 'Delete' button.
        
        This object handles POST requests the result from the user clicking on the 'Delete' button.  The
        response will be to delete the object selected in the table and redirect to the
        'base_url' URL.
        
        :Parameters
            - 'request': The request object passed in from the Django framework
            - 'info_msg' : A message to add into the session data at the key 'info_message' if the delete is 
            successful. The delete object will be appended to the end of the message. If nothing is supplied
            default message is "Deleted <obj>".
            
        :Returns
            - The return value of the django.http.HttpResponseRedirect object
            
        :Raises
            - Many different exceptions could be raised.
        '''
        try:
            if self.base_url:
                obj = self.modelClass.objects.get(id=request.POST['radioSelect'])
                obj.delete()
                try:
                    if info_msg:
                        request.session['info_messages'].append(_("%(m)s %(o)s") %{'m':info_msg, 'o':obj})
                    else:
                        request.session['info_messages'].append(_("Deleted %(o)s") %{'o':obj})
                except KeyError, e:
                    if info_msg:
                        request.session['info_messages'] = [_("%(m)s %(o)s") %{'m':info_msg, 'o':obj}]
                    else:
                        request.session['info_messages'] = [_("Deleted %(o)s") %{'o':obj}]
                return HttpResponseRedirect(self.base_url)
            else:
                raise Exception(self.noURLMsg)
        except KeyError, e:
            if self.base_url:
                request.session['info_messages'] = [_("Please select an item from the list")]
                return HttpResponseRedirect(self.base_url)
            else:
                raise Exception(self.noURLMsg)

    def addButton(self, request):
        '''Logic for handling POST request due to the user clicking the 'Add' button.
        
        This object handles POST requests the result from the user clicking on the 'Add' button.  The
        response will be to redirect to the /ovmtb/resources/yum/add URL.
        
        :Parameters
            - 'request': The request object passed in from the Django framework
            
        :Returns
            - The return value of the django.http.HttpResponseRedirect object
            
        :Raises
            - Many different exceptions could be raised.
        '''
        return HttpResponseRedirect(self.add_url)

    def __init__(self,
            modelClass=None,
            dict_key=None,
            context_params=None,
            context_keys=None,
            function_map=None,
            template=None,
            add_url=None,
            edit_url=None,
            base_url=None):
        '''Creates a TableFormPage object that will put a list of objects of modelClass type into
        the context_param dictionary at the key dict_key, and then displays the template.  POST
        logic is handled by mapping the POSTing object to a function via the function_map dictionary.

        One use method is to instantiate this class, passing in ALL of the desired parameters to the constructor,
        and then call it's 'HttpResponse' method with the request object passed in by the Django framework.

        This class can be used in shorthand by instantiating an instance with only some or none of the
        parameters, and then using the getFunction and postFunction rather than HttpResponse.  If it
        is used in this manner and a parameter is not supplied, an exception will be thrown.

        :Parameters
            - 'modelClass': The class of objects used to populate the table.  Should be a subclass
              of the django.db.models.Model class.
            - 'dict_key': The context variable to store the list of objects in.  Should be used in the
              template tags.
            - 'context_params': The context parameters passing into the template.
            - 'context_keys': Keys to look for in the session data and store in context_params
            - 'function_map': A dictionary mapping the "name" value of POSTing HTML objects to functions
              that handle the logic for that object.
            - 'template': The template to load.
            - 'add_edit_url': A URL to redirect to if the user clicks an 'Add' or Edit' button
            - 'base_url': A URL to redirect to if the user clicks the 'Delete' button or does something non-sensical
        '''
        self.modelClass = modelClass
        self.dict_key = dict_key
        self.context_params = context_params
        if context_keys:
            self.context_keys = context_keys
        else:
            self.context_keys = [('info_messages', 'info_messages'), ('error_messages', 'error_messages')]
        if function_map:
            self.function_map = function_map
        else:
            self.function_map = {
                'addButton': self.addButton,
                'deleteButton': self.deleteButton,
                'editButton': self.editButton
            }
        self.template = template
        self.add_url = add_url
        self.edit_url = edit_url
        self.base_url = base_url
        
        self.noModelClassMsg = _("The TableFormPage object was called without a modelClass, and none was")
        self.noModelClassMsg += _(" defined in the class level variable.  There has to be a modelClass")
        self.noModelClassMsg += _(" provided so the right list of objects is extracted from the database")
        self.noDictKeyMsg = _("The TableFormPage object was called without a dict_key, and none was")
        self.noDictKeyMsg += _(" defined in the class level variable.  There has to be a dicy_key")
        self.noDictKeyMsg += _(" provided so the list of objects can be stored at the key the template")
        self.noDictKeyMsg += _(" is expecting")
        self.noContextParams = _("The TableFormPage object was called without a context_params dictionary,")
        self.noContextParams += _(" and none was defined in the class level variable.  There has to be a")
        self.noContextParams += _(" context_params dictionary provided so the template is displayed properly")
        self.noFunctionMapMsg = _("The TableFormPage object was called without a function_map, and none was")
        self.noFunctionMapMsg += _(" defined in the class level variable.  There has to be a function_map")
        self.noFunctionMapMsg += _(" provided so the right function is called to handle the POST request")
        self.noTemplateMsg = _("The TableFormPage object was called without a template, and none was")
        self.noTemplateMsg += _(" defined in the class level variable.  There has to be a template provided")
        self.noTemplateMsg += _(" so something gets rendered")
        self.noURLMsg = _("The TableFormPage object was called without a URL to redirect to, and none was")
        self.noURLMsg += _(" defined in the class level variable.  There has to be a URL provided")
        self.noURLMsg += _(" so the user may be redirected")

    def __checkParams(self, type,
            modelClass=None,
            dict_key=None,
            context_params=None,
            function_map=None,
            template=None,
            add_url=None,
            edit_url=None,
            base_url=None):
        '''Internal function to validate parameters. Raises and Exception if any is invalid/missing.
        Should not need to be called externally.

        :Parameters
            - 'type': 'GET' or 'POST'
            - 'modelClass': Should not be None.
            - 'dict_key': Should be a String that is not None or ''.
            - 'context_params': Should be a dictionary that is not None.
            - 'function_map': Should be a dictionary that is not None.
            - 'template': Should be a String that is not None or ''.

        :Returns
            - a five tuple of the parameters or the value from the class level variable in the following
              order (modelClass, dict_key, context_params, function_map, template)

        :Raises
            - Exception: if any of the parameters is invalid, or if any parameter and the class level variable
              is not defined.
        '''
        mc = modelClass or self.modelClass
        if not mc: raise Exception(self.noModelClassMsg)
        dk = dict_key or self.dict_key
        if not dk: raise Exception(self.noDictKeyMsg)
        cp = context_params or self.context_params
        if not cp: raise Exception(self.noContextParamsMsg)
        fm = function_map or self.function_map
        if not fm and type != 'GET': raise Exception(self.noFunctionmapMsg)
        t = template or self.template
        if not t: raise Exception(self.noTemplateMsg)
        au = add_url or self.add_url
        if not au and type != 'GET': raise Exception(self.noURLMsg)
        eu = edit_url or self.edit_url
        if not eu and type != 'GET': raise Exception(self.noURLMsg)
        bu = template or self.template
        if not bu and type != 'GET': raise Exception(self.noURLMsg)

        return (mc, dk, cp, fm, t)

    def getFunction(self, request,
            modelClass=None,
            dict_key=None,
            context_params=None,
            template=None):
        '''View logic for GET requests

        This function holds the basic view logic for any GET request.  The default action will be
        to initialize a list of all instances of the modelClass object held in the database. The
        list will be stored in the context_params dictionary using the key dict_key. The 
        django.template.loader.get_template object will then be used to load the template, an
        instance of the django.template.RequestConext will be created, populated with the context_params
        dictionary, and request object.  The django.template.loader.render object will be called
        with the django.template.RequestContext and returned using django.http.HttpResponse.

        This is the standard but long winded way to display a view in the Django framework, For more
        information see the Django documentation:

        http://docs.djangoproject.com/en/dev/topics/http/views/#topics-http-views

        The template called should be written to access the list_key value from the context to populate
        the table (as well as define the table structure and buttons).

        :Parameters
            - 'request': The request object passed in from the Django framework.
            - 'modelClass': The class of objects used to populate the table.  Should be a subclass
              of the django.db.models.Model class.
            - 'dict_key': The context variable to store the list of objects in.  Should be used in the
              template tags.
            - 'context_params': The context parameters passing into the template.
            - 'template': The template to load.

        If a parameter is not passed in or passed as None the value of the class variable of the same
        name will be used.

        :Returns
            - the return of the django.http.HttpResponse class constructor.

        :Throws
            - This could throw many types of exceptions, don't have a complete list.
        '''
        (mc, dk, cp, fm, t,) = self.__checkParams('GET', modelClass=modelClass, dict_key=dict_key,
            context_params=context_params, template=template)

        list = mc.objects.all()
        l = loader.get_template(t)
        cp[dk] = list
        c = RequestContext(request, cp)
        return HttpResponse(l.render(c))

    def postFunction(self, request,
            modelClass=None,
            dict_key=None,
            context_params=None,
            function_map=None,
            template=None,
            add_url=None,
            edit_url=None,
            base_url=None):
        '''View logic for POST requests

        This function holds the basic view logic for any POST request.  The default action will be
        to call the function in function_map based on the HTML object contained in the POST data.
        It is up to the function called to do the right thing.

        :Parameters
            - 'request': The request object passed in from the Django framework.
            - 'modelClass': The class of objects used to populate the table.  Should be a subclass
              of the django.db.models.Model class.
            - 'dict_key': The context variable to store the list of objects in.  Should be used in the
              template tags.
            - 'context_params': The context parameters passing into the template.
            - 'function_map': A dictionary mapping the "name" value of POSTing HTML objects to functions
              that handle the logic for that object.
            - 'template': The template to load.

        If a parameter is not passed in or passed as None the value of the class variable of the same
        name will be used.

        :Returns
            - the return of the function that is called from function_map.

        :Throws
            - This could throw many types of exceptions, don't have a complete list.
        '''
        (mc, dk, cp, fm, t) = self.__checkParams('POST', modelClass=modelClass, dict_key=dict_key,
            context_params=context_params, template=template, add_url=add_url, edit_url=edit_url, base_url=base_url)
            
        for key in fm.keys():
            if key in request.POST:
                return fm[key](request)
        
        raise Exception(_('No defined function matched anything in the POST data'))

    def scanSessionData(self, request, keys, dict):
        '''Checks the session for data that should be stored in context_params or session_data.

        :Parameters
            - 'request': The request object passed in from the Django framework.
            - 'keys': A list of tuples containing the key to look for and the key
            to store the value under in the dictionary.
            - 'dict': The dictionary to store the session data in.
        '''
        for pair in keys:
            try:
                value = request.session[pair[0]]
                if not dict:
                    dict = {}
                dict[pair[1]] = value
                del request.session[pair[0]]
            except KeyError, e:
                continue

    def scanSessionDataForContextKeys(self, request):
        '''Checks the session for data that should be stored in context_params.

        :Parameters
            - 'request': The request object passed in from the Django framework.
        '''
        self.scanSessionData(request, self.context_keys, self.context_params)

    def HttpResponse(self, request):
        '''Entry point for processing all requests and displaying the template.

        This object will examine the request object passed in by the Django framework, and then call
        self.getFunction or self.postFunction as appropriate.

        :Parameters
            - 'request': The request object passed in from the Django framework.

        Using this object requires that all of the parameters were set using the constructor.  If not,
        this will result in an exception being thrown.

        :Returns
            - the return of django.http.HttpResponse if it is a GET request.
            - the return of the function that is called from function_map if it is a POST request.

        :Throws
            - This could throw many types of exceptions, don't have a complete list.
        '''
        self.scanSessionDataForContextKeys(request)

        if request.method == 'POST':
            return self.postFunction(request)
        else:
            return self.getFunction(request)

