# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import ugettext as _
from ovmtb.resources.TableFormPage import TableFormPage
from ovmtb.resources.models import YUMRepo
from ovmtb.templates.common import *
from subprocess import Popen, PIPE
import datetime, random

def cacheYUM(repo):
    '''Private object that caches the passed in YUM repository.
    
    This object should not be called external to the class.  Generally it is called by the cacheYUMButton object in order
    to generate the list of packages and groups available in the YUM repository

    :Parameters
        - 'repo': The repository to cache.
        
    :Returns
        - None
        
    :Raises
        - Many different exceptions could be raised.
    '''
    pkgs = []
    grps = []

    random.seed()
    suffix = random.uniform(1,1000000)
    repo_base_cmd = "repoquery --repofrompath=tmprepo.%d,%s --repoid=tmprepo.%d --tempcache -a " %(suffix, repo.url, suffix)
    repo_packages_cmd = repo_base_cmd + "--qf=\"%{NAME}.%{ARCH}\""
    repo_groups_cmd = repo_base_cmd + "-g"

    packageprocess = Popen(repo_packages_cmd, shell=True, bufsize=0, stdout=PIPE, stderr=PIPE, close_fds=False)
    out = packageprocess.stdout
    err = packageprocess.stderr
    for line in out.readlines()[1:]:
        if line.startswith("Cannot retrieve"):
            raise Exception(line)
        pkgs.append(line.strip())
    errors = err.read()
    if errors:
        raise Exception(errors)

    groupprocess = Popen(repo_groups_cmd, shell=True, bufsize=0, stdout=PIPE, stderr=PIPE, close_fds=False)
    out = groupprocess.stdout
    err = groupprocess.stderr
    for line in out.readlines()[1:]:
        grps.append(line.strip())

    repo.packages = " ".join(sorted(pkgs))
    repo.groups = " ".join(sorted(grps))

class YUMTableFormPage(TableFormPage):
    '''View logic for a page consisting of a table listing YUM repositories and action buttons.

    The TableFormPage class implements the basic view functionality for a page that will display a
    form that contains a table of YUM repositories.  Behavoir should be identical to TableFormPage
    except that the function_map has been defined in the constructor.
    '''
    
    def preventCascadingDelete(self, request):
        '''Normally when an object with a ForeignKey relationship is deleted,
        all foreign objects that where in the relationship will be deleted as
        well.  We do not want this to happen when a Yum repo is deleted.
        
        :Parameters
            - 'request': The Http request object passed in from the Django
            framework
        :Returns
            - None, but any TemplateProject that was using the repo will be
            modified.
        '''
        try:
            if self.base_url:
                obj = self.modelClass.objects.get(id=request.POST['radioSelect'])
                projects = obj.templateproject_set.all()
                for project in projects:
                    project.yum_repo = None
                    project.save()
                    log_info_message(request, _('Modified Template Project: %(t)s') % {'t':project})
            else:
                raise Exception(self.noURLMsg)
        except KeyError, e:
            if self.base_url:
                log_info_message(request, _("Please select an item from the list"))
                return HttpResponseRedirect(self.base_url)
            else:
                raise Exception(self.noURLMsg)
    
    def deleteButton(self, request):
        self.preventCascadingDelete(request)
        return TableFormPage.deleteButton(self, request, info_msg="Deleted Yum repository:")

    def cacheButton(self, request):
        '''Logic for handling POST request due to the user clicking the 'Cache' button.
        
        This object handles POST requests the result from the user clicking on the 'Cache' button.  The
        response will be to redirect to the /ovmtb/resources/yum/edit/<repo_id> URL.
        
        :Parameters
            - 'request': The request object passed in from the Django framework
            
        :Returns
            - The return value of the django.http.HttpResponseRedirect object
            
        :Raises
            - Many different exceptions could be raised.
        '''
        try:
            repo = self.modelClass.objects.get(id=request.POST['radioSelect'])
            cacheYUM(repo)
            repo.cached = datetime.datetime.now()
            repo.save()
            log_info_message(request, _("Cached YUM repository: %(r)s") %{'r':repo})
            return HttpResponseRedirect(self.base_url)
        except KeyError, e:
            if self.base_url:
                log_info_message(request, _("Please select an item from the list"))
                return HttpResponseRedirect(self.base_url)
            else:
                raise Exception(self.noURLMsg)
        except Exception, e:
            log_error_message(request, _("Could not cache YUM repository %(r)s: %(err)s") %{'r':repo, 'err':e})
            return HttpResponseRedirect(self.base_url)

    def __init__(self,
            modelClass=None,
            dict_key=None,
            context_params=None,
            template=None):
        '''Creates a YUMTableFormPage object.

        :Parameters
            - 'modelClass': The class of objects used to populate the table.  Should be a subclass
              of the django.db.models.Model class.
            - 'dict_key': The context variable to store the list of objects in.  Should be used in the
              template tags.
            - 'context_params': The context parameters passing into the template.
            - 'template': The template to load.
        '''
        TableFormPage.__init__(self, modelClass=modelClass, dict_key=dict_key, context_params=context_params, template=template,
            add_url='/ovmtb/resources/yum/add/', edit_url='/ovmtb/resources/yum/edit/',base_url='/ovmtb/resources/yum/')
        self.function_map['cacheButton'] = self.cacheButton
        self.function_map['deleteButton'] = self.deleteButton

