# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.forms import ModelForm, Form, FileField, CharField
from django.utils.translation import ugettext as _
from django.forms.widgets import Textarea
from django.conf import settings
from subprocess import Popen, PIPE
from ovmtb.templates.WizardStep import WizardStep
from ovmtb.templates.models import TemplateProject
from ovmtb.templates.common import *
import os, re

class DescriptionStep(WizardStep):
    '''The Description class implements the logic behind the 'OVM Template
    Project Description' page.
    '''
    
    def __init__(self,
            template = 'description.html',
            context_params = {
                'tab_template': "templates_tab_active.html",
                'submenu_template': "no_submenu.html",
            },
            cancel_url = '/ovmtb/templates/',
            save_url = '/ovmtb/templates/save/',
            previous_url = '/ovmtb/templates/',
            next_url = '/ovmtb/templates/hardware/'):
        self.template = template
        self.context_params = context_params
        self.cancel_url = cancel_url
        self.save_url = save_url
        self.previous_url = previous_url
        self.next_url = next_url

    def saveFormData(self, data, form):
        '''Apparently, when you call the save() method of a ModelForm class that
        has been defined to operate on a subset of the fields in the Model class
        the fields that are not included are also modified.  Since this will 
        result in user changes being lost this method is used instead of the 
        standard form.save(commit=False)
        
        The saveFormData method is a utility function that will take the data
        held in the cleaned_data attribute of the ModelForm class, and store it
        in the dictionary passed in as data.  Presumably this dictionary is then
        saved to the session data.

        Note that this method assumes that the data dictionary contains a
        TemplateProject object at the key 'TemplateProject' and that the form
        object passed in is a DescriptionForm object that has been validated.
        
        :Parameters
            - 'data': The dictionary to store the cleaned_data in.
            - 'form': Instance of the DescriptionForm class.
        '''
        project = data['TemplateProject']
        project.template_name = form.cleaned_data['template_name']
        project.version = form.cleaned_data['version']
        project.vendor = form.cleaned_data['vendor']
        project.JeOS_image = form.cleaned_data['JeOS_image']
        project.description = form.cleaned_data['description']
        project.EULA = form.cleaned_data['EULA']
    
    def generateResponse(self, request, project_id=None):
        # Load the data from the session
        data = loadWork(request, project_id=project_id)

        if request.method == 'POST':
            if 'cancelButton2' in request.POST or 'cancelButton1' in request.POST:
                clearWork(request, data=data, project_id=project_id)
                return HttpResponseRedirect(self.cancel_url)

            # Now create a bound form
            form = DescriptionForm(request.POST, data['TemplateProject'])

            if form.is_valid():
                self.saveFormData(data, form)
                if 'removeEULAButton' in request.POST:
                    data['TemplateProject'].EULA = ""
                saveWork(request, project_id=project_id, data=data)
            else:
                self.context_params['form'] = form
                t = loader.get_template(self.template)
                get_messages(request, self.context_params)
                c = RequestContext(request, self.context_params)
                saveWork(request, project_id=project_id, data=data)
                return HttpResponse(t.render(c))
            
            if 'removeEULAButton' in request.POST:
                base_url = '/ovmtb/templates/description/'
            elif 'addEULAButton' in request.POST:
                base_url = '/ovmtb/templates/description/add_EULA/'
            elif 'saveButton2' in request.POST or 'saveButton1' in request.POST: 
                base_url = self.save_url
            elif 'nextButton2' in request.POST or 'nextButton1' in request.POST:
                base_url = self.next_url

            saveWork(request, project_id=project_id, data=data)
            return HttpResponseRedirect(map_url(base_url, project_id))

        else: # request.method == 'GET'
            self.context_params['form'] = DescriptionForm(instance=data['TemplateProject'])
            t = loader.get_template(self.template)
            get_messages(request, self.context_params)
            c = RequestContext(request, self.context_params)
            saveWork(request, project_id=project_id, data=data)
            return HttpResponse(t.render(c))

class DescriptionForm(ModelForm):
    version = CharField(required=False, label="Template version")
    EULA = CharField(required=False, label="End User License Agreement", widget=Textarea)

    class Meta:
        model = TemplateProject
        fields = ('template_name', 'version', 'vendor', 'JeOS_image', 'description', 'EULA')
