# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
from django.template import RequestContext, Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.forms import ModelForm, IntegerField, Select, ValidationError
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _
from ovmtb.templates.models import HardwareDescription, TemplateProject, DiskImage
from ovmtb.templates.common import *
from ovmtb.templates.WizardStep import WizardStep
import datetime, logging, re

class HardwareDisksStep(WizardStep):
    def __init__(self,
            template = 'hardware_disks.html',
            context_params = {
                'tab_template': "templates_tab_active.html",
                'submenu_template': "no_submenu.html",
            },
            cancel_url = '/ovmtb/templates/hardware/',
            save_url = '/ovmtb/templates/hardware/'):
        self.template = template
        self.context_params = context_params
        self.cancel_url = cancel_url
        self.save_url = save_url

    def generateResponse(self, request, disk_id, type, project_id=None):
        data = loadWork(request, project_id=project_id)

        if request.method == 'POST':
            if 'cancelButton2' in request.POST or 'cancelButton1' in request.POST:
                # There are two cases we need to handle here.  First is a disk that
                # has been saved, and the user wants to cancel their edits.  Second
                # is a disk that has not been saved and the user wants to cancel
                # the addition of the disk to the project.
                if type == 'edit':
                    # Just drop the edits and return
                    saveWork(request, project_id=project_id, data=data)
                    return HttpResponseRedirect(map_url(self.cancel_url, project_id))
                elif type == 'add':
                    # The user wants to cancel the addition of the disk.
                    data['disk_list'].pop(int(disk_id))
                    saveWork(request, project_id=project_id, data=data)
                    return HttpResponseRedirect(map_url(self.cancel_url, project_id))
            
            form = HardwareDisksForm(request.POST, instance=data['disk_list'][int(disk_id)])
            form.setDiskList(data['disk_list'])
            if form.is_valid():
                if 'saveButton2' in request.POST or 'saveButton1' in request.POST:
                    disk = form.save(commit=False)
                    saveWork(request, project_id=project_id, data=data)
                    return HttpResponseRedirect(map_url(self.save_url, project_id))
            else:
                self.context_params['form'] = form
                t = loader.get_template(self.template)
                c = RequestContext(request, self.context_params)
                saveWork(request, project_id=project_id, data=data)
                return HttpResponse(t.render(c))

        else:
            form = HardwareDisksForm(instance=data['disk_list'][int(disk_id)])
            self.context_params['form'] = form
            t = loader.get_template(self.template)
            get_messages(request, self.context_params)
            c = RequestContext(request, self.context_params)
            saveWork(request, project_id=project_id, data=data)
            return HttpResponse(t.render(c))

class HardwareDisksForm(ModelForm):

    def clean_size(self):
        '''Customized validation to ensure that the size parameter is at least 512
        '''
        data = int(self.cleaned_data['size'])
        if data < 512:
            raise ValidationError(_("The disk should be at least 512MB"))
        return data

    def clean_mount_point(self):
        '''Customized validation to ensure that the mount point path starts with '/'
        and that no other disk images are mounted on the same mount point.
        '''
        data = self.cleaned_data['mount_point']
        if data:
            if not re.match('/', data):
                raise ValidationError(_("Path to mount point should be absolute (starts with /)"))

            for disk in self.disk_list:
                if disk is self.instance:
                    continue
                elif disk.mount_point and disk.mount_point == data:
                    raise ValidationError(_("%(d)s is already mounted at this path") %{'d':disk.disk_image_file})

        return data

    def clean_disk_image_file(self):
        '''Customized validation to ensure that the disk image filename is unique in all of the
        DiskImage objects associated with this HardwareDescription.
        '''
        data = self.cleaned_data['disk_image_file']

        if not data:
            raise ValidationError(_("You need to supply a file name for the disk image"))

        for disk in self.disk_list:
            if disk is self.instance:
                continue
            elif disk.disk_image_file == data:
                raise ValidationError(_("There is another disk named %(d)s, pleae choose another name") %{'d':data})

        return data

    def setDiskList(self, disk_list):
        '''There is no good way to determine the list of related DiskImage objects so that the
        mount_point and disk_image_file fields can be validated as unique.  This method allows
        the view logic to manually pass in the disk list.

        :Parameters
            - 'disk_list' : A list of DiskImage objects associated with the same
            HardwareDescription object.
        '''
        self.disk_list = disk_list

    class Meta:
        model = DiskImage
        fields = ('size', 'disk_image_file', 'mount_point')
