# Copyright (C) 2009 Oracle Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
from django.contrib.auth.decorators import login_required
from django.conf import settings
from ovmtb.templates.models import *
import random, os, shutil

def cleanUpUploadDir(data):
    '''This method will clean up the upload directory of a project.  This is 
    intended to be used for the case where a user edits/creates a template,
    uploading packages, and then Cancels out of the wizard.

    More specifically this method will go through the dictionary passed in
    and examine the TemplateProject object.  If the object has not been saved
    to the database, the entire upload directory will be removed.  If it has
    been saved, then each of the UserPackage objects will be examined, and if
    they have not been saved the delete() method will be called to remove the
    file.

    :Parameters
        - 'data': A dictionary matching the structure described in the
        loadProjectIntoSession method documentation.
    :Returns
        - None, but the filesystem used to hold the upload directory will be
        altered.
    '''
    project = data['TemplateProject']

    # if project.id is None, the object has not been saved to the database,
    # so just remove the upload directory
    if not project.id:
        shutil.rmtree(project.upload_dir, ignore_errors=True)
    # Project has been saved, so we do not want to remove the entire upload
    # directory, but we need to go through the package list and delete any
    # that have not been saved.
    else:
        package_list = data['package_list']
        package_list.extend(data['deleted_packages'])
        for package in package_list:
            if not package.id:
                os.remove(package.package_file._get_path())

def checkDir(dir):
    '''Checks to see if a directory exists.  If it does not, then attempt to create it.
        
    :Parameters
        - 'dir': The directory to check.
    :Returns
        - True : if the directory did not exist and creation succeeded
        - False : if the directory exists, or creation fails
        
    Note that this method also has the side effect of creating the directory if
    it does not exist.
    '''
    if not os.access(dir, os.F_OK):
        try:
            os.mkdir(dir)
            return True
        except Exception, e:
            return False
    return False

def generateRandomDir():
    '''Generates a random 20 character string for use as a path to save the
    uploaded file.
    
    :Returns
        - A 20 character long string made of of lower and upper case letters and
         the digits 0-9
    '''
    dir=""
    for i in range(0,19):
        dir += random.choice("abcdefghijklmpopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789")
    return dir

@login_required
def get_messages(request, context_params):
    '''Loads any messages stored in the 'error_messages' or 'info_messages' keys
    in the request's session dictionary into the same key in the context_param
    dictionary passed into the method
    
    :Parameters
        - 'request': The HTML request object passed in from the Django framework
        - 'context_params': A dictionary into which the messages should be
        stored under the same key
    :Returns
        - None, but the dictionary passed in as context_params will be modified
        if the request's session dictionary contains the keys.
    '''
    try:
        context_params['error_messages'] = request.session['error_messages']
        del request.session['error_messages']
    except KeyError:
        pass
    
    try:
        context_params['info_messages'] = request.session['info_messages']
        del request.session['info_messages']
    except KeyError:
        pass

def map_url(base_url, project_id=None):
    '''Simple function that returns a url suffixed with the project_id if
    project_id is not None.  Seems simple, but this logic was happening over
    and over again, so I refactored it out.
    
    :Parameters
        - 'base_url': The prefix of the URL to be returned.  Note that this 
	should end with a slash '/'.
        - 'project_id': The id to append to the URL if not None
    :Returns
        - The base_url with project_id appended if project_id is not None and 
        base_url otherwise.
    '''
    if project_id:
        return "%s%s/" %(base_url, project_id)
    else:
        return base_url

@login_required    
def log_info_message(request, msg):
    '''Logs a message into the request's session dictionary at the key
    'info_messages'.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'msg': The message to log
    '''
    if 'info_messages' in request.session:
        request.session['info_messages'].append(msg)
    else:
        request.session['info_messages'] = [msg]

@login_required
def log_error_message(request, msg):
    '''Logs a message into the request's session dictionary at the key
    'error_messages'.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'msg': The message to log
    '''
    if 'error_messages' in request.session:
        request.session['error_messages'].append(msg)
    else:
        request.session['error_messages'] = [msg]

@login_required
def getEditTemplateFromSession(request, project_id):
    '''Loads the data for the project whose database id matches project_id from
    the request's session dictionary.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'project_id': The database ID of the TemplateProject object to get
        from the session dictionary
    :Returns
        - A dictionary containing the data for the project matching project_id
        in the format that it was stored under by loadProjectIntoSession if the
        project_id matches, and None otherwise.
        
    Note that the data is removed from the session dictionary if found.  Any
    class that calls this method that wants to store the data should call the 
    storeEditProjectIntoSession method.
    '''
    if 'EditTemplate' in request.session:
        et =  request.session['EditTemplate']
        if project_id in et:
            ret = et[project_id]
            del request.session['EditTemplate'][project_id]
            return ret
    return None

@login_required
def storeEditTemplateIntoSession(request, project_id, data):
    '''Stores the data associated with a TemplateProject object that is being
    edited into the request's session dictionary in the format described in the
    loadProjectIntoSession method. 

    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'project_id': The database ID of the TemplateProject object to store
        into the session dictionary
        - 'data': The data associated with the TemplateProject object.  Should 
        match what is described in loadProjectIntoSession.
    :Returns
        - None
    '''
    if not 'EditTemplate' in request.session:
        request.session['EditTemplate'] = {}
    request.session['EditTemplate'][project_id] = data
    request.session.modified = True

@login_required
def getNewTemplateFromSession(request):
    '''Retrieves the data associated with a new TemplateProject object that is
    being edited but has not been saved into the database.

    :Parameters
        - 'request': The HTML request object passed in by the Django framework
    :Returns
        - A dictionary containing the data for the project matching project_id
        in the format that it was stored under by newProjectInSession if
        the project_id matches, and None otherwise.
        
    Note that the data is removed from the session dictionary if found.  Any
    class that calls this method that wants to store the data should call the 
    storeNewProjectIntoSession method.
    '''
    if 'NewTemplate' in request.session:
        ret = request.session['NewTemplate']
        del request.session['NewTemplate']
        return ret
    return None

@login_required
def storeNewTemplateIntoSession(request, data):
    '''Store data into the request's session dictionary for a new template.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'data': The data associated with the TemplateProject object.  Should 
        match what is described in newProjectInSession.
    :Returns
        - None
    
    Note that any data in the session dictionary at the 'NewTemplate' key will
    be overwritten.
    '''
    request.session['NewTemplate'] = data
    request.session.modified = True
        
@login_required
def loadProjectIntoSession(request, project_id):
    '''This method will load the project from the database, along with all
    associated HardwareDescription, DiskImage, and UserPackage objects into the
    request's session dictionary at the key 'EditTemplate'.  The value stored
    at this key will be a dictionary, containing a key matching the id of the
    TemplateProject to be edited.  The value at the key matching the
    TemplateProject will be a dictionary containing the following key/value
    pairs:

        - 'TemplateProject': The TemplateProject object from the database
        - 'HardwareDescription': The HardwareDescription object from the
        database that is associated with the TemplateProject
        - 'disk_list': A list of DiskImage objects that belong to the
        HardwareDescription object associated with the TemplateProject
        - 'package_list': A list of UserPackage objects that are associated
        with the TemplateProject object
        - 'deleted_disks': A list of DiskImage objects that the user has chosen
        to remove from the HardwareDescription
        - 'deleted_packages': A list of UserPackage objects that the user has
        chosen to remove from the TemplateProject  
    
    So a rough diagram of the data structure is as follows
        'EditTemplate'
            --> 'ID'
                --> 'TemplateProject'
                --> 'HardwareDescription'
                --> 'disk_list'
                --> 'package_list'
                --> 'deleted_disks'
                --> 'deleted_packages'
    
    All classes involved in the wizard should manipulate the data structure in
    the session data and allow the save method to actually update the objects in
    the database.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'project_id': The database ID of the TemplateProject object to load
        into the session dictionary
    :Returns
        - None, but the request.session dictionary will be modified.  Generally
        this means that database IO will be involved.
    '''
    if not 'EditTemplate' in request.session:
        request.session['EditTemplate'] = {}

    p = TemplateProject.objects.get(id=project_id)
    h = HardwareDescription.objects.get(template_project__id__exact=project_id)
    disks = []
    packages = []
    for d in DiskImage.objects.filter(hardware_id__id__exact=h.id):
        disks.append(d)
    for pack in UserPackage.objects.filter(template_project__id__exact=project_id):
        packages.append(pack)
    et = { 'TemplateProject': p,
        'HardwareDescription': h,
        'disk_list': disks,
        'package_list': packages,
        'deleted_disks': [],
        'deleted_packages': [],
    }
    request.session['EditTemplate'][project_id] = et
    request.session.modified = True
    
@login_required
def newProjectInSession(request):
    '''The newProjectInSession method will populate the request's session
    dictionary with a data structure containing all the objects that will be
    part of the new project definition.
    
    The structure will be stored at the key 'NewTemplate' in the request's 
    session dictionary.  A dictionary will be stored as the value at that key.
    The dictionary will have the following key/values:
    
        - 'TemplateProject': An unsaved TemplateProject object
        - 'HardwareDescription': An unsaved HardwareDescription object
        - 'disk_list': A list of DiskImage objects that will be part of the
        template.
        - 'package_list': A list of UserPackage objects that will be part of the
        template.
        
    So a rough diagram of the data structure is as follows
        'NewTemplate'
            --> 'TemplateProject'
            --> 'HardwareDescription'
            --> 'disk_list'
            --> 'package_list'
            --> 'deleted_disks'
            --> 'deleted_packages'

    All classes involved in the wizard should manipulate the data structure in
    the session data and allow the save method to actually update the objects in
    the database.
    
    Note that anything at 'NewTemplate' will be deleted, so any existing project
    definition in progress will be erased.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
    :Returns
        - None, but the request.session dictionary will be modified.  Generally
        this means that database IO will be involved.
    '''
    p =  TemplateProject()
    p.owner = request.user
    
    save_dir = '%s/%s' %(settings.OVMTB_UPLOAD_DIR, generateRandomDir())
    while not checkDir(save_dir):
        save_dir = '%s/%s' %(settings.OVMTB_UPLOAD_DIR, generateRandomDir())
    p.upload_dir = save_dir
    
    nt = { 'TemplateProject': p,
        'HardwareDescription': HardwareDescription(template_project=p),
        'disk_list': [],
        'package_list': [],
        'deleted_disks': [],
        'deleted_packages': [],
    }
    request.session['NewTemplate'] = nt
    
@login_required
def loadWork(request, project_id=None):
    '''Loads data from the session.  If project_id is None the data associated
    with a new TemplateProject is returned.  Otherwise the data associated with
    the TemplateProject identified by project_id is returned.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'project_id': The database ID of the TemplateProject object to load.
    '''
    if not project_id:
        ret = getNewTemplateFromSession(request)
        if not ret:
            newProjectInSession(request)
            ret = getNewTemplateFromSession(request)
        return ret
    else:
        ret = getEditTemplateFromSession(request, project_id)
        if not ret:
            loadProjectIntoSession(request, project_id)
            ret = getEditTemplateFromSession(request, project_id)
        return ret

@login_required
def saveWork(request, project_id=None, data={}):
    '''Saves data associated with a TemplateProject into the session dictionary.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'project_id': The database ID of the TemplateProject object to save.
        - 'data': The data to save.  Should match what is described in either
        the newProjectInSession or loadProjectIntoSession.
    :Returns
        - None
    '''
    if not project_id:
        storeNewTemplateIntoSession(request, data)
    else:
        storeEditTemplateIntoSession(request, project_id, data)

@login_required
def clearWork(request, data=None, project_id=None):
    '''Cancels any changes made to a TemplateProject that are stored in the 
    request's session dictionary.
    
    :Parameters
        - 'request': The HTML request object passed in by the Django framework
        - 'data': A datastruture representing a TemplateProject that has been
        pulled out of the request's session info.
        - 'project_id': The database ID of the TemplateProject object to clear.
    :Returns
        - None
    '''
    if data:
        cleanUpUploadDir(data)

    if not project_id:
        if 'NewTemplate' in request.session:
            del request.session['NewTemplate']
    else:
        if 'EditTemplate' in request.session:
            if project_id in request.session['EditTemplate']:
                del request.session['EditTemplate'][project_id]
    request.session.modified = True
